#!/usr/bin/perl
#
# check_KAMP.pl - nagios plugin
#
# Copyright (C) 2012 B1 Systems GmbH <info@b1-systems.de>,
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
#
## purpose: This Nagios Plugin checks a Kentix AlarmManager-Pro. 
## author: Christian Schneemann <schneemann@b1-systems.de>, 2012.

use strict;
use warnings;

use Nagios::Plugin;
use Net::SNMP;

my $VERSION = "1.0.0";

my $OIDdef = "1.3.6.1.4.1.37954.1.1";

my %moduletypes = ( alarmzone1          => 1,
                    alarmzone2          => 2,
                    alarmzone3          => 3,
                    alarm1              => 4,
                    alarm2              => 5,
                    serverstate         => 6,
                    sensorcommunication => 7,
                    extalarm            => 8,
                    extarmed            => 9,
                    extpower            => 10,
                    sabotage            => 11,
                    gsmsignal           => 12,
                    gsmok               => 13,
                  );

my %moduletypes_id2string;
for my $n (keys %moduletypes) {
    $moduletypes_id2string{$moduletypes{$n}} = $n;
}

my $np = Nagios::Plugin->new(
    usage => "Usage: %s [ -v|--verbose ]  -H <host> -C <communitystring> -m <module>[-t <timeout>] 
              [ -e|--expects=<expected value> ] ",
    version => $VERSION,
    shortname => "check_KAMP",
    blurb   => "This Plugin checks a Kentix Alarmmanager-Pro.",
#    extra   => $extra,
#    url     => $url,
    license => "GPLv2",
#    plugin  => basename $0,
    timeout => 15,
);

$np->add_arg(
    spec => 'host|H=s',
    help => "-H, --host=STRING\n   hostname or IP of Kentix Alarmmanager-Pro",
    required => 1,
    );

$np->add_arg(
    spec => 'community|C=s',
    help => "-C, --community=STRING\n   community string for snmp connection",
    required => 1,
    );

$np->add_arg(
    spec => 'module|m=s',
    help => "-m, --module=STRING\n   module to check: alarmzone1, alarmzone2, alarmzone3, alarm1, alarm2, serverstate,
   sensorcommunication, extalarm, extarmed, extpower, sabotage, gsmsignal, gsmok",
    required => 1,
    );

$np->add_arg(
    spec => 'expects|e=i',
    help => "-e, --expects=INTEGER\n   expected value if checking alarmzone1, alarmzone2, alarmzone3, alarm1, alarm2, serverstate,
   sensorcommunication, extalarm, extarmed, extpower or sabotage",
    required => 0,
    );

$np->add_arg(
    spec => 'warning|w=i',
    help => "-w, --warning=INTEGER\n   warning threshold",
    required => 0,
    );

$np->add_arg(
    spec => 'critical|c=i',
    help => "-c, --critical=INTEGER\n   critical threshold",
    required => 0,
    );

$np->getopts;

my ($module, $snmp_host, $snmp_community, $snmp_timeout, $snmp_version);
my (@oids, @exitmessage, @warnings, @criticals);
my (%values);
my $rc = "OK";

$module = $np->opts->get('module');
$snmp_host = $np->opts->get('host');
$snmp_community = $np->opts->get('community');
$snmp_timeout = $np->opts->get('timeout');
$snmp_version = "2c";

if ( grep{$module eq $_} ("alarmzone1", "alarmzone2", "alarmzone3", "alarm1", "alarm2", "serverstate",
                          "sensorcommunication", "extalarm", "extarmed", "extpower", "sabotage",
                          "gsmsignal", "gsmok", "gsm" )) {
    #\@oids 
    if ($module eq "gsm") {
        push( @oids, "$OIDdef.$moduletypes{'gsmsignal'}.0"); # get value for one module
        push( @oids, "$OIDdef.$moduletypes{'gsmok'}.0"); # get value for one module

    } else {
        push( @oids, "$OIDdef.$moduletypes{$module}.0"); # get value for one module
    }
} else {
    $np->nagios_die("unkown module\n");
}


# alarm is set in snmp session
#alarm $np->opts->timeout;

my ($session, $error) = Net::SNMP->session( -hostname => $snmp_host,
                                            -community => $snmp_community,
                                            -version => $snmp_version,
                                            -timeout => $snmp_timeout,
                                          );
if (!defined ($session)) {
    $np->nagios_exit(3, $error);
}

my $res = $session->get_request( -varbindlist => \@oids,);

$session->close();

if (!$res) {
    $np->nagios_exit(3, "no data received from $snmp_host\n");
}

for my $key (keys %$res) {
    $key =~ /$OIDdef\.(\d+)\.0/;
    my $module = $1;
    
    $values{$moduletypes_id2string{$module}} = $res->{$key};     
}

# These modules just returns 0 or 1, so no real warning/critical handling is possible
if ( grep{$module eq $_} ("alarmzone1", "alarmzone2", "alarmzone3", "alarm1", "alarm2", "serverstate",
                          "sensorcommunication", "extalarm", "extarmed", "extpower", "sabotage",
                          "gsmok", "gsm") ) {
    my $mod = $module;
    $mod = "gsmok" if $module eq "gsm";
    push(@exitmessage, "$mod: ".$values{$mod});

    if ( defined $np->opts->get("expects")) {
        if ($np->opts->get("expects") == 1 || $np->opts->get("expects") == 0 ) {
            if ( $values{$mod} != $np->opts->get("expects") ) {
                push(@criticals, "$mod is CRITICAL, should be ", $np->opts->get("expects"));
            }
        }
    }
} 

if ( $module eq "gsmsignal" || $module eq "gsm") {
    if (defined $values{'gsmsignal'}) {

        push(@exitmessage, "gsmsignal: ".$values{'gsmsignal'});

        my ($w, $c);
        $w = $np->opts->get("warning");
        $c = $np->opts->get("critical");
        my $check = 0;
        if ($values{'gsmsignal'} <= $w ) {
            $check = 1;
        }
        if ($values{'gsmsignal'} <= $c ) {
            $check = 2;
        }

        if ($check == 1) {
            push(@warnings, "$values{'gsmsignal'} is WARNING,");
        } elsif ($check == 2) {
            push(@criticals, "$values{'gsmsignal'} is CRITICAL,");
        } 
        $np->add_perfdata( label    => "gsmsignal",
                           value    => $values{'gsmsignal'},
                           warning  => $w,
                           critical => $c,
                          );
    }
}



if (@warnings) {
    $rc = "WARNING";
    unshift(@exitmessage, join(" ", @warnings));
} 
if (@criticals) {
    $rc = "CRITICAL";
    unshift(@exitmessage, join(" ", @criticals));
}
$np->nagios_exit($rc, join(", ", @exitmessage) );

