#!/usr/bin/perl
#
# check_KAMP_sensors.pl - nagios plugin
#
# Copyright (C) 2012 B1 Systems GmbH <info@b1-systems.de>,
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
#
## purpose: This Nagios Plugin checks the sensors connected 
##          to a Kentix AlarmManager Pro. 
## author: Christian Schneemann <schneemann@b1-systems.de>, 2012.

use strict;
use warnings;

use Nagios::Plugin;
use Net::SNMP;

my $VERSION = "1.0.0";

my $OIDdef = "1.3.6.1.4.1.37954.1.2";

my %sensortypes = ( temperature => 2,
                    humidity    => 3,
                    dewpoint    => 4,
                    co          => 5,
                  );
# what about motion => 6?


my %sensortypes_id2string;
$sensortypes_id2string{'1'} = "sensorname";
for my $n (keys %sensortypes) {
    $sensortypes_id2string{$sensortypes{$n}} = $n;
}

my $np = Nagios::Plugin->new(
    usage => "Usage: %s [ -v|--verbose ]  -H <host> -C <communitystring> [-s <sensor>][-t <timeout>]
              [ -c|--critical=<critical threshold> ] [ -w|--warning=<warning threshold> ]  ",
    version => $VERSION,
    shortname => "check_KAMP_sensors",
    blurb   => "This Plugin checks the sensors connected to a Kentix AlarmManager Pro.",
#    extra   => $extra,
#    url     => $url,
    license => "GPLv2",
#    plugin  => basename $0,
    timeout => 15,
);

$np->add_arg(
    spec => 'host|H=s',
    help => "--host, -H\n   hostname or IP of Kentix Alarmmanager Pro",
    required => 1,
    );

$np->add_arg(
    spec => 'community|C=s',
    help => "--community, -C\n   community string for snmp connection",
    required => 1,
    );

$np->add_arg(
    spec => 'sensor|s=i',
    help => "--sensor, -s\n   number of sensor to check, if not set all sensors are checked",
    default => "all",
    );

$np->add_arg(
    spec => 'sensortype|t=s',
    help => "--sensortype, -t\n   number of sensor to check, if not set all sensors are checked
   valid types: temperature, humidity, dewpoint, co",
    default => "all",
    );

#$np->add_arg(
#    spec => 'warning|w=s',
#    help => "--warning, -w ",
#    required => 0,
#    );

#$np->add_arg(
#    spec => 'critical|c=s',
#    help => "--critical, -w ",
#    required => 0,
#    );

$np->add_arg(
    spec => 'warning-temperature=s',
    help => "--warning-temperature\n   warning value for temperature",
    required => 0,
    );

$np->add_arg(
    spec => 'critical-temperature=s',
    help => "--critical-temperature\n   critical value for temperature",
    required => 0,
    );

$np->add_arg(
    spec => 'warning-humidity=s',
    help => "--warning-humidity\n   warning value for humidity",
    required => 0,
    );

$np->add_arg(
    spec => 'critical-humidity=s',
    help => "--critical-humidity\n   critical value for humidity",
    required => 0,
    );

$np->add_arg(
    spec => 'warning-dewpoint=s',
    help => "--warning-dewpoint\n   warning value for dewpoint",
    required => 0,
    );

$np->add_arg(
    spec => 'critical-dewpoint=s',
    help => "--critical-dewpoint\n   critical value for dewpoint",
    required => 0,
    );

$np->add_arg(
    spec => 'warning-co=s',
    help => "--warningco\n   warning value for co",
    required => 0,
    );

$np->add_arg(
    spec => 'critical-co=s',
    help => "--criticalco\n   critical value for co",
    required => 0,
    );

$np->getopts;

if (defined $np->opts->get('warning-temperature') && !defined $np->opts->get('critical-temperature') || 
    !defined $np->opts->get('warning-temperature') && defined $np->opts->get('critical-temperature') ||
    defined $np->opts->get('warning-humidity') && !defined $np->opts->get('critical-humidity') || 
    !defined $np->opts->get('warning-humidity') && defined $np->opts->get('critical-humidity') ||
    defined $np->opts->get('warning-dewpoint') && !defined $np->opts->get('critical-dewpoint') || 
    !defined $np->opts->get('warning-dewpoint') && defined $np->opts->get('critical-dewpoint') ||
    defined $np->opts->get('warning-co') && !defined $np->opts->get('critical-co') || 
    !defined $np->opts->get('warning-co') && defined $np->opts->get('critical-co') || 
    defined $np->opts->get('warning') && !defined $np->opts->get('critical') || 
    !defined $np->opts->get('warning') && defined $np->opts->get('critical') ) {

    $np->nagios_die("warning(-*) and critical(-*) needs to be given as a pair\n");
} 


my ($sensor, $sensortype, $warning, $critical, $snmp_host, $snmp_community, $snmp_timeout, $snmp_version);
my (@oids, @exitmessage, @warnings, @criticals);
my (%values);
my $rc = "OK";

$sensor = $np->opts->get('sensor');
$sensortype = $np->opts->get('sensortype');
$warning = $np->opts->get('warning');
$critical = $np->opts->get('critical');
$snmp_host = $np->opts->get('host');
$snmp_community = $np->opts->get('community');
$snmp_timeout = $np->opts->get('timeout');
$snmp_version = "2c";

#\@oids 
push( @oids, "$OIDdef.$sensor.1.0"); # get name
if ($sensortype eq "all") {
# for motion let count till 6
    for my $j (2..5) {
        # get value for all sensortypes
        push( @oids, "$OIDdef.$sensor.$j.0");
    }
} else {
    push( @oids, "$OIDdef.$sensor.$sensortypes{$sensortype}.0"); # get value for one sensortype
}

# alarm is set in snmp session
#alarm $np->opts->timeout;

my ($session, $error) = Net::SNMP->session( -hostname => $snmp_host,
                                            -community => $snmp_community,
                                            -version => $snmp_version,
                                            -timeout => $snmp_timeout,
                                          );
if (!defined ($session)) {
    $np->nagios_exit(4, $error);
}

my $res = $session->get_request( -varbindlist => \@oids,);

$session->close();

if (!$res) {
    $np->nagios_exit(4, "no data received from $snmp_host\n");
}

for my $key (keys %$res) {
    $key =~ /$OIDdef.(\d).(\d).0/;
    my $sensor = $1;
    my $type = $2;

    $values{$sensortypes_id2string{$type}} = $res->{$key};     
}

my %uom = ( temperature => "°C", humidity => "", dewpoint => "", co => "");

for my $value ("temperature", "humidity", "dewpoint", "co") {
    if (defined $values{$value}) {
        my $val = $values{$value};
        $val = $val/10 if $value ne "co";

        push(@exitmessage, "$value: ".$values{$value}/10);
 
        if (defined $np->opts->get("warning-$value") && defined $np->opts->get("critical-$value")) {
            my $check = $np->check_threshold(check => $val,
                                 warning => $np->opts->get("warning-$value"),
                                 critical => $np->opts->get("critical-$value")
                                );
            if ($check == 1) {
                push(@warnings, "$value is WARNING");
            } elsif ($check == 2) {
                push(@criticals, "$value is CRITICAL");
            } 
            $np->add_perfdata( label => $value,
                               value => $val,
                               uom => $uom{$value},
                               warning => $np->opts->get("warning-$value"),
                               critical => $np->opts->get("critical-$value"),
                             );
        } else {
            $np->add_perfdata( label => $value, value => $val, uom => $uom{$value}, );
        }
    }
}

if (@warnings) {
    $rc = "WARNING";
    unshift(@exitmessage, join(" ", @warnings));
} 
if (@criticals) {
    $rc = "CRITICAL";
    unshift(@exitmessage, join(" ", @criticals));
}
unshift(@exitmessage, "$values{'sensorname'}");
$np->nagios_exit($rc, join(", ", @exitmessage) );

